@@ -27,9 +27,6 @@ from py2048 import Directions, type_check
 from py2048.grid import Grid
 
 
-DIRECTIONS_TUPLE = tuple(Directions)
-
-
 class Base2048Frontend(ABC):
     """This abstract class implements the `play2048` method. A concrete
     frontend should inherit from this class, but shouldn't override this
@@ -41,6 +38,7 @@ class Base2048Frontend(ABC):
     # 2^11 == 2048
     # sys.maxsize == (2^63) - 1 on 64bits machines
     valid_numbers: List[int] = [2 ** power for power in range(1, 12)]
+    DIRECTIONS = tuple(Directions)
     SLEEP_S = 1  # how many seconds to sleep when in auto mode
     # converting seconds to mseconds
     SLEEP_MS = int(SLEEP_S * 1_000)
@@ -57,9 +55,15 @@ class Base2048Frontend(ABC):
         self.grid.reset()
         self.grid.seed()
 
-    def __init__(self, grid: Grid, *, goal: Optional[int] = None):
+    def __init__(
+        self,
+        grid: Grid,
+        is_random: bool,
+        *,  # makes the remaining arguments keyword-only
+        goal: Optional[int] = None,
+    ) -> None:
+        # first, perform some checks
         type_check(grid, Grid)
-        self.grid = grid
         # `goal = goal or 2048` would allow "goal = 0" to pass silently
         if goal is None:
             goal = 2048
@@ -67,7 +71,14 @@ class Base2048Frontend(ABC):
             raise OverflowError(f"the value {goal} is too big of a goal")
         elif not self.is2048like(goal):
             raise ValueError("goal must be a positive power of 2")
+        # checks passed
+        self.grid = grid
         self.goal = goal
+        if is_random:
+            self.choice_function = self.guess_direction
+        else:
+            self.choice_function = self.choose_direction
+        self.is_random = is_random
         self.victory = False
 
     # play hooks: don't need to be overriden, but probably should
@@ -93,7 +104,7 @@ class Base2048Frontend(ABC):
         pass
 
     def guess_direction(self) -> Directions:
-        return random.choice(DIRECTIONS_TUPLE)
+        return random.choice(self.DIRECTIONS)
 
     # MAIN LOOP: shouldn't be overriden
     @final
@@ -127,7 +138,7 @@ class Base2048Frontend(ABC):
                 # Ctrl-D or quit-command
                 player_quit = True
                 break
-            assert choice in DIRECTIONS_TUPLE
+            assert choice in self.DIRECTIONS
             self.after_choice(choice)
             dragged = grid.drag(choice)
             if dragged:
@@ -135,9 +146,8 @@ class Base2048Frontend(ABC):
                 jammed = grid.jammed
             else:
                 self.after_nochange(choice)
-            grid.attempt += 1
             self.after_attempt(choice)
-            if not self.victory and grid.biggest >= self.goal:
+            if not self.victory and grid.largest_number >= self.goal:
                 self.victory = True
                 break
         # after loop stuff
@@ -158,9 +168,9 @@ class Base2048Frontend(ABC):
         self.player_loss()
 
     # the following methods MUST be overriden
-    def choice_function(self) -> Directions:
+    def choose_direction(self) -> Directions:
         raise NotImplementedError(
-            "'choice_function' must be overriden by subclass"
+            "'choose_direction' must be overriden by subclass"
         )
 
     def player_quit(self) -> Any:
